@@ -1,15 +1,15 @@
 # Copyright (C) 2011  Lars Wirzenius
-# 
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License along
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
@@ -22,18 +22,24 @@ import re
 class ManpageGenerator(object):
 
     '''Fill in a manual page template from an OptionParser instance.'''
-    
+
     def __init__(self, template, parser, arg_synopsis, cmd_synopsis):
         self.template = template
         self.parser = parser
         self.arg_synopsis = arg_synopsis
         self.cmd_synopsis = cmd_synopsis
-        
+
+    def sort_options(self, options):
+        return sorted(options,
+                      key=lambda o: (o._long_opts + o._short_opts)[0])
+
+    def option_list(self, container):
+        return self.sort_options(container.option_list)
+
     @property
     def options(self):
-        return sorted(self.parser.option_list,
-                      key=lambda o: (o._long_opts + o._short_opts)[0])
-                      
+        return self.option_list(self.parser)
+
     def format_template(self):
         sections = (('SYNOPSIS', self.format_synopsis()),
                     ('OPTIONS', self.format_options()))
@@ -42,13 +48,16 @@ class ManpageGenerator(object):
             pattern = '\n.SH %s\n' % section
             text = text.replace(pattern, pattern + contents)
         return text
-                      
+
     def format_synopsis(self):
         lines = []
         lines += ['.nh']
         lines += ['.B %s' % self.esc_dashes(self.parser.prog)]
-        
-        for option in self.options:
+
+        all_options = self.option_list(self.parser)
+        for group in self.parser.option_groups:
+            all_options += self.option_list(group)
+        for option in self.sort_options(all_options):
             for spec in self.format_option_for_synopsis(option):
                 lines += ['.RB [ %s ]' % spec]
 
@@ -65,7 +74,7 @@ class ManpageGenerator(object):
 
         lines += ['.hy']
         return ''.join('%s\n' % line for line in lines)
-                      
+
     def format_option_for_synopsis(self, option):
         if option.metavar:
             suffix = '\\fR=\\fI%s' % self.esc_dashes(option.metavar)
@@ -75,8 +84,17 @@ class ManpageGenerator(object):
             yield '%s%s' % (self.esc_dashes(name), suffix)
 
     def format_options(self):
-        return ''.join(self.format_option_for_options(option)
-                       for option in self.options)
+        lines = []
+
+        for option in self.sort_options(self.parser.option_list):
+            lines += self.format_option_for_options(option)
+
+        for group in self.parser.option_groups:
+            lines += ['.SS "%s"' % group.title]
+            for option in self.sort_options(group.option_list):
+                lines += self.format_option_for_options(option)
+
+        return ''.join('%s\n' % line for line in lines)
 
     def format_option_for_options(self, option):
         lines = []
@@ -90,8 +108,8 @@ class ManpageGenerator(object):
                      for x in option._long_opts]
         lines += ['.BR ' + ' ", " '.join(shorts + longs)]
         lines += [self.esc_dots(self.expand_default(option).strip())]
-        return ''.join('%s\n' % line for line in lines)
-        
+        return lines
+
     def expand_default(self, option):
         default = self.parser.defaults.get(option.dest)
         if default is optparse.NO_DEFAULT or default is None:
@@ -99,16 +117,16 @@ class ManpageGenerator(object):
         else:
             default = str(default)
         return option.help.replace('%default', default)
-        
+
     def esc_dashes(self, optname):
         return '\\-'.join(optname.split('-'))
-    
+
     def esc_dots(self, line):
         if line.startswith('.'):
             return '\\' + line
         else:
             return line
-            
+
     def format_argspec(self, argspec):
         roman = re.compile(r'[^A-Z]+')
         italic = re.compile(r'[A-Z]+')
