@@ -1,15 +1,15 @@
 # Copyright (C) 2009-2012  Lars Wirzenius
-# 
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License along
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
@@ -24,6 +24,18 @@ import sys
 import cliapp
 from cliapp.genman import ManpageGenerator
 
+
+log_group_name = 'Logging'
+config_group_name = 'Configuration files and settings'
+perf_group_name = 'Peformance'
+
+default_group_names = [
+    log_group_name,
+    config_group_name,
+    perf_group_name,
+]
+
+
 class Setting(object):
 
     action = 'store'
@@ -31,25 +43,27 @@ class Setting(object):
     nargs = 1
     choices = None
 
-    def __init__(self, names, default, help, metavar=None, group=None):
+    def __init__(
+        self, names, default, help, metavar=None, group=None, hidden=False):
         self.names = names
         self.set_value(default)
         self.help = help
         self.metavar = metavar or self.default_metavar()
         self.group = group
+        self.hidden = hidden
 
     def default_metavar(self):
         return None
 
     def get_value(self):
         return self._string_value
-        
+
     def set_value(self, value):
         self._string_value = value
-        
+
     def call_get_value(self):
         return self.get_value()
-        
+
     def call_set_value(self, value):
         self.set_value(value)
 
@@ -74,9 +88,11 @@ class StringSetting(Setting):
 class StringListSetting(Setting):
 
     action = 'append'
-    
-    def __init__(self, names, default, help, metavar=None, group=None):
-        Setting.__init__(self, names, [], help, metavar=metavar, group=group)
+
+    def __init__(
+        self, names, default, help, metavar=None, group=None, hidden=False):
+        Setting.__init__(
+            self, names, [], help, metavar=metavar, group=group, hidden=hidden)
         self.default = default
         self.using_default_value = True
 
@@ -88,17 +104,17 @@ class StringListSetting(Setting):
             return [s.strip() for s in self._string_value.split(',')]
         else:
             return self.default
-        
+
     def set_value(self, strings):
         self._string_value = ','.join(strings)
         self.using_default_value = False
 
     def has_value(self):
         return self.value != []
-        
+
     def parse_value(self, string):
         self.value = [s.strip() for s in string.split(',')]
-        
+
     def format(self): # pragma: no cover
         return ', '.join(self.value)
 
@@ -106,16 +122,18 @@ class StringListSetting(Setting):
 class ChoiceSetting(Setting):
 
     type = 'choice'
-    
-    def __init__(self, names, choices, help, metavar=None, group=None):
-        Setting.__init__(self, names, choices[0], help, metavar=metavar,
-                         group=group)
+
+    def __init__(
+        self, names, choices, help, metavar=None, group=None, hidden=False):
+        Setting.__init__(
+            self, names, choices[0], help, metavar=metavar, group=group,
+            hidden=hidden)
         self.choices = choices
 
     def default_metavar(self):
         return self.names[0].upper()
 
-    
+
 class BooleanSetting(Setting):
 
     action = 'store_true'
@@ -127,7 +145,7 @@ class BooleanSetting(Setting):
 
     def get_value(self):
         return self._string_value.lower() in self._trues
-        
+
     def set_value(self, value):
         def is_true():
             if value is True or value is False:
@@ -145,8 +163,8 @@ class ByteSizeSetting(Setting):
 
     def parse_human_size(self, size):
         '''Parse a size using suffix into plain bytes.'''
-        
-        m = re.match(r'''(?P<number>\d+(\.\d+)?) \s* 
+
+        m = re.match(r'''(?P<number>\d+(\.\d+)?) \s*
                          (?P<unit>k|ki|m|mi|g|gi|t|ti)? b? \s*$''',
                      size.lower(), flags=re.X)
         if not m:
@@ -171,7 +189,7 @@ class ByteSizeSetting(Setting):
 
     def get_value(self):
         return long(self._string_value)
-        
+
     def set_value(self, value):
         if type(value) == str:
             value = self.parse_human_size(value)
@@ -187,7 +205,7 @@ class IntegerSetting(Setting):
 
     def get_value(self):
         return long(self._string_value)
-        
+
     def set_value(self, value):
         self._string_value = str(value)
 
@@ -196,22 +214,10 @@ class FormatHelpParagraphs(optparse.IndentedHelpFormatter):
 
     def _format_text(self, text): # pragma: no cover
         '''Like the default, except handle paragraphs.'''
-        
-        def format_para(lines):
-            para = '\n'.join(lines)
-            return optparse.IndentedHelpFormatter._format_text(self,  para)
-        
-        paras = []
-        cur = []
-        for line in text.splitlines():
-            if line.strip():
-                cur.append(line)
-            elif cur:
-                paras.append(format_para(cur))
-                cur = []
-        if cur:
-            paras.append(format_para(cur))
-        return '\n\n'.join(paras)
+
+        fmt = cliapp.TextFormat(width=self.width)
+        formatted = fmt.format(text)
+        return formatted.rstrip('\n')
 
 
 class Settings(object):
@@ -220,40 +226,40 @@ class Settings(object):
 
     You probably don't need to create a settings object yourself,
     since ``cliapp.Application`` does it for you.
-    
+
     Settings are read from configuration files, and parsed from the
     command line. Every setting has a type, name, and help text,
     and may have a default value as well.
-    
+
     For example::
-    
+
         settings.boolean(['verbose', 'v'], 'show what is going on')
-        
+
     This would create a new setting, ``verbose``, with a shorter alias
     ``v``. On the command line, the options ``--verbose`` and
-    ``-v`` would work equally well. There can be any number of aliases. 
+    ``-v`` would work equally well. There can be any number of aliases.
 
     The help text is shown if the user uses ``--help`` or
     ``--generate-manpage``.
     You can use the ``metavar`` keyword argument to set the name shown
     in the generated option lists; the default name is whatever
     ``optparse`` decides (i.e., name of option).
-    
+
     Use ``load_configs`` to read configuration files, and
     ``parse_args`` to parse command line arguments.
-    
+
     The current value of a setting can be accessed by indexing
     the settings class::
-    
+
         settings['verbose']
 
     The list of configuration files for the appliation is stored
     in ``config_files``. Add or remove from the list if you wish.
     The files need to exist: those that don't are silently ignored.
-    
+
     '''
 
-    def __init__(self, progname, version, usage=None, description=None, 
+    def __init__(self, progname, version, usage=None, description=None,
                  epilog=None):
         self._settingses = dict()
         self._canonical_names = list()
@@ -263,44 +269,50 @@ class Settings(object):
         self.usage = usage
         self.description = description
         self.epilog = epilog
-        
+
         self._add_default_settings()
-        
+
         self._config_files = None
         self._cp = ConfigParser.ConfigParser()
 
     def _add_default_settings(self):
-        self.string(['output'], 
+        self.string(['output'],
                     'write output to FILE, instead of standard output',
                     metavar='FILE')
 
-        self.string(['log'], 
+        self.string(['log'],
                     'write log entries to FILE (default is to not write log '
                         'files at all); use "syslog" to log to system log, '
                         'or "none" to disable logging',
-                    metavar='FILE')
-        self.choice(['log-level'], 
+                    metavar='FILE', group=log_group_name)
+        self.choice(['log-level'],
                     ['debug', 'info', 'warning', 'error', 'critical', 'fatal'],
                     'log at LEVEL, one of debug, info, warning, '
                         'error, critical, fatal (default: %default)',
-                    metavar='LEVEL')
-        self.bytesize(['log-max'], 
+                    metavar='LEVEL', group=log_group_name)
+        self.bytesize(['log-max'],
                       'rotate logs larger than SIZE, '
                         'zero for never (default: %default)',
-                      metavar='SIZE', default=0)
+                      metavar='SIZE', default=0, group=log_group_name)
         self.integer(['log-keep'], 'keep last N logs (%default)',
-                     metavar='N', default=10)
-        self.string(['log-mode'], 
+                     metavar='N', default=10, group=log_group_name)
+        self.string(['log-mode'],
                     'set permissions of new log files to MODE (octal; '
                         'default %default)',
-                    metavar='MODE', default='0600')
+                    metavar='MODE', default='0600', group=log_group_name)
 
         self.choice(['dump-memory-profile'],
                     ['simple', 'none', 'meliae', 'heapy'],
                     'make memory profiling dumps using METHOD, which is one '
                         'of: none, simple, meliae, or heapy '
                         '(default: %default)',
-                    metavar='METHOD')
+                    metavar='METHOD',
+                    group=perf_group_name)
+        self.integer(['memory-dump-interval'],
+                     'make memory profiling dumps at least SECONDS apart',
+                     metavar='SECONDS',
+                     default=300,
+                     group=perf_group_name)
 
     def _add_setting(self, setting):
         '''Add a setting to self._cp.'''
@@ -315,10 +327,10 @@ class Settings(object):
 
     def string_list(self, names, help, default=None, **kwargs):
         '''Add a setting which have multiple string values.
-        
+
         An example would be an option that can be given multiple times
         on the command line, e.g., "--exclude=foo --exclude=bar".
-        
+
         '''
 
         self._add_setting(StringListSetting(names, default or [], help,
@@ -326,12 +338,12 @@ class Settings(object):
 
     def choice(self, names, possibilities, help, **kwargs):
         '''Add a setting which chooses from list of acceptable values.
-        
+
         An example would be an option to set debugging level to be
         one of a set of accepted names: debug, info, warning, etc.
-        
+
         The default value is the first possibility.
-        
+
         '''
 
         self._add_setting(ChoiceSetting(names, possibilities, help, **kwargs))
@@ -342,11 +354,11 @@ class Settings(object):
 
     def bytesize(self, names, help, default=0, **kwargs):
         '''Add a setting with a size in bytes.
-        
+
         The user can use suffixes for kilo/mega/giga/tera/kibi/mibi/gibi/tibi.
-        
+
         '''
-        
+
         self._add_setting(ByteSizeSetting(names, default, help, **kwargs))
 
     def integer(self, names, help, default=0, **kwargs):
@@ -361,7 +373,7 @@ class Settings(object):
 
     def __contains__(self, name):
         return name in self._settingses
-        
+
     def __iter__(self):
         '''Iterate over canonical settings names.'''
         for name in self._canonical_names:
@@ -373,21 +385,21 @@ class Settings(object):
 
     def require(self, name):
         '''Raise exception if setting has not been set.
-        
+
         Option must have a value, and a default value is OK.
-        
+
         '''
-        
+
         if not self._settingses[name].has_value():
             raise cliapp.AppException('Setting %s has no value, '
                                         'but one is required' % name)
-        
+
     def _option_names(self, names):
         '''Turn setting names into option names.
-        
+
         Names with a single letter are short options, and get prefixed
         with one dash. The rest get prefixed with two dashes.
-        
+
         '''
 
         return ['--%s' % name if len(name) > 1 else '-%s' % name
@@ -398,11 +410,25 @@ class Settings(object):
         return name
 
     def build_parser(self, configs_only=False, arg_synopsis=None,
-                     cmd_synopsis=None):
+                     cmd_synopsis=None, deferred_last=[], all_options=False):
         '''Build OptionParser for parsing command line.'''
 
+        # Call a callback function unless we're in configs_only mode.
         maybe = lambda func: (lambda *args: None) if configs_only else func
 
+
+        # Maintain lists of callback function calls that are deferred.
+        # We call them ourselves rather than have OptionParser call them
+        # directly so that we can do things like --dump-config only
+        # after the whole command line is parsed.
+
+        def defer_last(func): # pragma: no cover
+            def callback(*args):
+                deferred_last.append(lambda: func(*args))
+            return callback
+
+        # Create the command line parser.
+
         def getit(x):
             if x is None or type(x) in [str, unicode]:
                 return x
@@ -415,41 +441,77 @@ class Settings(object):
                                   usage=usage,
                                   description=description,
                                   epilog=self.epilog)
-        
+
+        # Create all OptionGroup objects. This way, the user code can
+        # add settings to built-in option groups.
+
+        group_names = set(default_group_names)
+        for name in self._canonical_names:
+            s = self._settingses[name]
+            if s.group is not None:
+                group_names.add(s.group)
+        group_names = sorted(group_names)
+
+        option_groups = {}
+        for name in group_names:
+            group = optparse.OptionGroup(p, name)
+            p.add_option_group(group)
+            option_groups[name] = group
+
+        config_group = option_groups[config_group_name]
+
+        # Return help text, unless setting/option is hidden, in which
+        # case return optparse.SUPPRESS_HELP.
+
+        def help_text(text, hidden):
+            if all_options or not hidden:
+                return text
+            else:
+                return optparse.SUPPRESS_HELP
+
+        # Add --dump-setting-names.
+
         def dump_setting_names(*args): # pragma: no cover
             for name in self._canonical_names:
                 sys.stdout.write('%s\n' % name)
             sys.exit(0)
 
-        p.add_option('--dump-setting-names',
+        config_group.add_option('--dump-setting-names',
                      action='callback',
                      nargs=0,
-                     callback=maybe(dump_setting_names),
-                     help='write out all names of settings and quit')
+                     callback=defer_last(maybe(dump_setting_names)),
+                     help=help_text(
+                        'write out all names of settings and quit', True))
+
+        # Add --dump-config.
 
         def call_dump_config(*args): # pragma: no cover
             self.dump_config(sys.stdout)
             sys.exit(0)
 
-        p.add_option('--dump-config',
+        config_group.add_option('--dump-config',
                      action='callback',
                      nargs=0,
-                     callback=maybe(call_dump_config),
+                     callback=defer_last(maybe(call_dump_config)),
                      help='write out the entire current configuration')
 
+        # Add --no-default-configs.
+
         def reset_configs(option, opt_str, value, parser):
             self.config_files = []
 
-        p.add_option('--no-default-configs',
+        config_group.add_option('--no-default-configs',
                      action='callback',
                      nargs=0,
                      callback=reset_configs,
                      help='clear list of configuration files to read')
 
+        # Add --config.
+
         def append_to_configs(option, opt_str, value, parser):
             self.config_files.append(value)
 
-        p.add_option('--config',
+        config_group.add_option('--config',
                      action='callback',
                      nargs=1,
                      type='string',
@@ -457,16 +519,20 @@ class Settings(object):
                      help='add FILE to config files',
                      metavar='FILE')
 
+        # Add --list-config-files.
+
         def list_config_files(*args): # pragma: no cover
             for filename in self.config_files:
                 print filename
             sys.exit(0)
 
-        p.add_option('--list-config-files',
+        config_group.add_option('--list-config-files',
                      action='callback',
                      nargs=0,
-                     callback=maybe(list_config_files),
-                     help='list all possible config files')
+                     callback=defer_last(maybe(list_config_files)),
+                     help=help_text('list all possible config files', True))
+
+        # Add --generate-manpage.
 
         self._arg_synopsis = arg_synopsis
         self._cmd_synopsis = cmd_synopsis
@@ -475,9 +541,29 @@ class Settings(object):
                      nargs=1,
                      type='string',
                      callback=maybe(self._generate_manpage),
-                     help='fill in manual page TEMPLATE',
+                     help=help_text('fill in manual page TEMPLATE', True),
                      metavar='TEMPLATE')
 
+        # Add --help-all.
+
+        def help_all(*args): # pragma: no cover
+            pp = self.build_parser(
+                configs_only=configs_only,
+                arg_synopsis=arg_synopsis,
+                cmd_synopsis=cmd_synopsis,
+                all_options=True)
+            sys.stdout.write(pp.format_help())
+            sys.exit(0)
+
+        config_group.add_option(
+            '--help-all',
+            action='callback',
+            help='show all options',
+            callback=defer_last(maybe(help_all)))
+
+        # Add other options, from the user-defined and built-in
+        # settingses.
+
         def set_value(option, opt_str, value, parser, setting):
             if setting.action == 'append':
                 if setting.using_default_value:
@@ -490,91 +576,110 @@ class Settings(object):
                 assert setting.action == 'store'
                 setting.value = value
 
+        def set_false(option, opt_str, value, parser, setting):
+            setting.value = False
+
         def add_option(obj, s):
             option_names = self._option_names(s.names)
-            obj.add_option(*option_names, 
+            obj.add_option(*option_names,
                            action='callback',
                            callback=maybe(set_value),
                            callback_args=(s,),
                            type=s.type,
                            nargs=s.nargs,
                            choices=s.choices,
-                           help=s.help,
+                           help=help_text(s.help, s.hidden),
                            metavar=s.metavar)
-        
+
+        def add_negation_option(obj, s):
+            option_names = self._option_names(s.names)
+            long_names = [x for x in option_names if x.startswith('--')]
+            neg_names = ['--no-' + x[2:] for x in long_names]
+            unused_names = [x for x in neg_names
+                            if x[2:] not in self._settingses]
+            obj.add_option(*unused_names,
+                           action='callback',
+                           callback=maybe(set_false),
+                           callback_args=(s,),
+                           type=s.type,
+                           help=help_text('', s.hidden))
+
+        # Add options for every setting.
+
         for name in self._canonical_names:
             s = self._settingses[name]
             if s.group is None:
-                add_option(p, s)
-                p.set_defaults(**{self._destname(name): s.value})
+                obj = p
+            else:
+                obj = option_groups[s.group]
 
-        groups = {}
-        for name in self._canonical_names:
-            s = self._settingses[name]
-            if s.group is not None:
-                groups[s.group] = groups.get(s.group, []) + [(name, s)]
-
-        groupnames = sorted(groups.keys())
-        for groupname in groupnames:
-            group = optparse.OptionGroup(p, groupname)
-            p.add_option_group(group)
-            for name, s in groups[groupname]:
-                add_option(group, s)
-                p.set_defaults(**{self._destname(name): s.value})
+            add_option(obj, s)
+            if type(s) is BooleanSetting:
+                add_negation_option(obj, s)
+            p.set_defaults(**{self._destname(name): s.value})
 
         return p
 
     def parse_args(self, args, parser=None, suppress_errors=False,
                     configs_only=False, arg_synopsis=None,
-                    cmd_synopsis=None):
+                    cmd_synopsis=None, compute_setting_values=None,
+                    all_options=False):
         '''Parse the command line.
-        
+
         Return list of non-option arguments. ``args`` would usually
         be ``sys.argv[1:]``.
-        
+
         '''
 
+        deferred_last = []
+
         p = parser or self.build_parser(configs_only=configs_only,
                                         arg_synopsis=arg_synopsis,
-                                        cmd_synopsis=cmd_synopsis)
+                                        cmd_synopsis=cmd_synopsis,
+                                        deferred_last=deferred_last,
+                                        all_options=all_options)
 
         if suppress_errors:
             p.error = lambda msg: sys.exit(1)
 
         options, args = p.parse_args(args)
+        if compute_setting_values: # pragma: no cover
+            compute_setting_values(self)
+        for callback in deferred_last: # pragma: no cover
+            callback()
         return args
 
     @property
     def _default_config_files(self):
         '''Return list of default config files to read.
-        
+
         The names of the files are dependent on the name of the program,
         as set in the progname attribute.
-        
+
         The files may or may not exist.
-        
+
         '''
-        
+
         configs = []
-        
+
         configs.append('/etc/%s.conf' % self.progname)
         configs += self._listconfs('/etc/%s' % self.progname)
         configs.append(os.path.expanduser('~/.%s.conf' % self.progname))
         configs += self._listconfs(
                         os.path.expanduser('~/.config/%s' % self.progname))
-        
+
         return configs
 
     def _listconfs(self, dirname, listdir=os.listdir):
         '''Return list of pathnames to config files in dirname.
-        
+
         Config files are expectd to have names ending in '.conf'.
-        
-        If dirname does not exist or is not a directory, 
+
+        If dirname does not exist or is not a directory,
         return empty list.
-        
+
         '''
-        
+
         if not os.path.isdir(dirname):
             return []
 
@@ -591,7 +696,7 @@ class Settings(object):
 
     def _set_config_files(self, config_files):
         self._config_files = config_files
-        
+
     config_files = property(_get_config_files, _set_config_files)
 
     def set_from_raw_string(self, name, raw_string):
@@ -602,9 +707,9 @@ class Settings(object):
 
     def load_configs(self, open=open):
         '''Load all config files in self.config_files.
-        
+
         Silently ignore files that do not exist.
-        
+
         '''
 
         cp = ConfigParser.ConfigParser()
@@ -636,7 +741,13 @@ class Settings(object):
         sys.exit(0)
 
     def as_cp(self):
-        '''Return a ConfigParser instance with current values of settings.'''
+        '''Return a ConfigParser instance with current values of settings.
+
+        Any sections outside of ``[config]`` are preserved as is. This
+        lets the application use those as it wishes, and assign any
+        meanings it desires to the section names.
+
+        '''
         cp = ConfigParser.ConfigParser()
         cp.add_section('config')
         for name in self._canonical_names:
